'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionCreator = actionCreator;
exports.asyncActionCreator = asyncActionCreator;
exports.createTypes = createTypes;
exports.async = async;

var _normalizr = require('normalizr');

var _detectNode = require('detect-node');

var _detectNode2 = _interopRequireDefault(_detectNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var SUCCESS = 'SUCCESS';
var FAIL = 'FAIL';

function actionCreator(type) {
  for (var _len = arguments.length, params = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    params[_key - 1] = arguments[_key];
  }

  return function () {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return params.reduce(function (action, param, index) {
      action.payload[param] = args[index];
      return action;
    }, { type: type, payload: {} });
  };
}

function asyncActionCreator(type, config) {
  if (typeof config === 'function') config = { action: config }; // eslint-disable-line no-param-reassign
  var _config = config,
      client = _config.client,
      server = _config.server,
      schema = _config.schema;


  return function (payload) {
    return function (dispatch) {
      for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
        args[_key3 - 1] = arguments[_key3];
      }

      var action = config.action || (_detectNode2.default ? server : client);
      dispatch({ type: type, payload: payload });
      return action.apply(undefined, [payload, dispatch].concat(args)).then(function (response) {
        return dispatch({
          type: type + '_' + SUCCESS,
          payload: payload,
          response: schema ? (0, _normalizr.normalize)(response, schema) : response
        });
      }, function (_ref) {
        var message = _ref.message,
            code = _ref.code;
        return dispatch({ type: type + '_' + FAIL, payload: payload, error: { message: message, code: code } });
      });
    };
  };
}

function createTypes(types, namespace) {
  return types.reduce(function (typeMap, type) {
    typeMap[type] = namespace ? namespace + '_' + type : type;
    return typeMap;
  }, {});
}

function async(type) {
  return [type, type + '_' + SUCCESS, type + '_' + FAIL];
}